@@ -226,8 +226,7 @@ return t, otherwise nil."
 
 (defun mew-fancy-summary-remove-invisible ()
   "Remove the invisible hook in Mew."
-  (when mew-gemacs-p
-    (jit-lock-unregister 'mew-summary-cook-region))
+  (jit-lock-unregister 'mew-summary-cook-region)
   (remove-hook 'window-scroll-functions 'mew-summary-cook-window 'local)
   (remove-hook 'pre-idle-hook 'mew-summary-cook-window))
 
@@ -284,10 +283,6 @@ return t, otherwise nil."
     (font-lock-unfontify-buffer)
     (font-lock-turn-off-thing-lock)))
 
-(when mew-xemacs-p
-  (defadvice mew-summary-folder-cache-load (after fancy-summary activate)
-    (font-lock-mode 1)))
-
 (defun mew-fancy-summary-thread-enable ()
   "Activate mew-fancy-summary in thread buffer."
   (when (mew-thread-p)
@@ -295,19 +290,6 @@ return t, otherwise nil."
     (setq mew-fancy-summary-scan-form
 		  (mew-get-summary-form (substring (mew-summary-folder-name 'ext) 1)))
     (setq mew-fancy-summary-thread-column (mew-vinfo-get-column))
-    ;; XEmacs + lazy-lock ???
-    (when mew-xemacs-p
-      (let* ((win (selected-window))
-	     (beg (window-start win))
-	     (end (window-end win t)))
-	(save-excursion
-	  (mew-fancy-summary-region
-	   (progn (goto-char beg)
-		  (forward-line (- (/ (frame-height) 2)))
-		  (point))
-	   (progn (goto-char end)
-		  (forward-line (/ (frame-height) 2))
-		  (point))))))
     (when (boundp 'font-lock-function)
       (set (make-local-variable 'font-lock-function)
 	    'mew-fancy-summary-font-lock-function))
