@@ -10,10 +10,8 @@
 (defvar mew-logo nil)
 
 (defconst mew-icon-mew "Mew.png")
-(defconst mew-icon-mew-mono "Mew.xbm")
-(defconst mew-icon-mew-mule-bitmap-image "Mew.img")
 
-(defconst mew-hello-message 
+(defconst mew-hello-message
 "
 
 Welcome to Mew world.
@@ -22,7 +20,7 @@ Mew -- Messaging in the Emacs World
 
 %s
 
-Copyright (C) 1994-2007 Kazu Yamamoto
+Copyright (C) 1994-2008 Kazu Yamamoto
 
 Please send comments to Kazu@Mew.org.
 
@@ -39,7 +37,7 @@ Please send comments to Kazu@Mew.org.
  "/|\\ - \\\\/"  "//\\ - \\\\/" "/-\\ - \\\\/" "/\\\\ - \\\\/"
  "/|\\ - \\\\/"  "//\\ - \\\\/"  "/-\\ - \\\\/" "/\\\\ - \\\\/"
 
- "/\\| - \\\\/" "/\\/ - \\\\/" "/\\- - \\\\/" "/\\\\ - \\\\/" 
+ "/\\| - \\\\/" "/\\/ - \\\\/" "/\\- - \\\\/" "/\\\\ - \\\\/"
  "/\\| - \\\\/" "/\\/ - \\\\/" "/\\- - \\\\/" "/\\\\ - \\\\/"
 
  "/\\\\ - |\\/" "/\\\\ - /\\/" "/\\\\ - -\\/" "/\\\\ - \\\\/"
@@ -64,23 +62,11 @@ Please send comments to Kazu@Mew.org.
     (if (null mew-demo-picture)
 	(mew-hello-text time)
       (cond
-       ((and mew-xemacs-p window-system
-	     (valid-image-instantiator-format-p 'png))
-	(goto-char (point-max))
-	(mew-flet
-	 (setq mew-logo
-	       (make-glyph (vector 'png ':file 
-				   (expand-file-name mew-icon-mew
-						     mew-icon-directory)))))
-	(indent-to (startup-center-spaces mew-logo))
-	(set-extent-end-glyph (mew-overlay-make (point) (point)) mew-logo)
-	(goto-char (point-min))
-	(mew-redraw time))
-       ((and mew-gemacs-p window-system
+       ((and window-system
 	     (fboundp 'image-type-available-p)
 	     (image-type-available-p 'png))
 	(goto-char (point-max))
-	(setq mew-logo (create-image 
+	(setq mew-logo (create-image
 			(expand-file-name mew-icon-mew mew-icon-directory)))
 	(setq indent (/ (- (window-width)
 			   (truncate (car (image-size mew-logo)))) 2))
@@ -89,19 +75,6 @@ Please send comments to Kazu@Mew.org.
 	(insert-image mew-logo)
 	(goto-char (point-min))
 	(mew-redraw time))
-       ((and (featurep 'bitmap) window-system)
-	(or
-	 (condition-case nil
-	     (mew-insert-file-contents
-	      (expand-file-name
-	       mew-icon-mew-mule-bitmap-image mew-icon-directory))
-	   (file-error nil))
-	 (condition-case nil
-	     (bitmap-insert-xbm-file
-	      (expand-file-name mew-icon-mew-mono mew-icon-directory))
-	   (file-error nil)))
-	(center-region (point-min) (point-max))
-	(mew-redraw time))
        (t
 	(mew-hello-text time)))
       (set-buffer-modified-p nil))))
@@ -116,17 +89,16 @@ Please send comments to Kazu@Mew.org.
   (if (and mew-demo window-system) (mew-demo)))
 
 (defun mew-demo (&optional string)
-  (let* ((list (or string mew-demo-string))
+  (let* ((strs (or string mew-demo-string))
 	 (wl (window-width))
 	 (ul (length (regexp-quote "/\\ - \\/")))
 	 (pl (/ (- wl ul) 2))
 	 (pre (make-string pl mew-sp))
 	 (suf (make-string (1- (- (- wl pl) ul)) mew-sp)))
     (save-window-excursion
-      (while list
-	(mew-demo-print (car list) pre suf)
-	(mew-demo-loop)
-	(setq list (cdr list))))))
+      (dolist (str strs)
+	(mew-demo-print str pre suf)
+	(mew-demo-loop)))))
 
 (defun mew-demo-print (string prefix suffix)
   (goto-char (point-max))
@@ -142,13 +114,13 @@ Please send comments to Kazu@Mew.org.
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -157,7 +129,7 @@ Please send comments to Kazu@Mew.org.
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
