@@ -32,19 +32,9 @@ Execute mew-dinfo-set before calling this."
 ;;;
 
 (defun mew-header-mode ()
-  "\\<mew-header-mode-map> Mew Header mode:: major mode for
-composing a MIME message by editing its header only.
+  "A major mode for composing a MIME message by editing its header only.
 
-To send the header and its body, type \\[mew-header-make-message] and \\[mew-header-send-message].
-
-Key assignment:
-
-\\[mew-header-make-message]	Compose a MIME message from the header and its body
-	then put it into a queue folder.
-\\[mew-header-send-message]	Compose a MIME message from the header and its body then send it.
-\\[mew-draft-set-case]	Guess case and set the case for output to it.
-\\[mew-draft-kill]	Kill this draft.
-"
+\\{mew-header-mode-map}"
   (interactive)
   (mew-draft-set-local-variables)
   (setq major-mode 'mew-header-mode)
@@ -60,13 +50,15 @@ Key assignment:
     (setq auto-fill-function 'mew-draft-auto-fill)))
 
 (defmacro mew-summary-header-mode (fields &rest body)
+  ;; (declare (indent 1))
   `(mew-summary-msg-or-part
     (mew-summary-display 'redisplay)
     (mew-current-set-window-config)
     (let* ((draft (mew-folder-new-message mew-draft-folder))
 	   (attachdir (mew-attachdir draft))
-	   (fld (mew-current-get-fld (mew-frame-id)))
-	   (msg (mew-current-get-msg (mew-frame-id))))
+	   (fid (mew-frame-id))
+	   (fld (mew-current-get-fld fid))
+	   (msg (mew-current-get-msg fid)))
       (mew-summary-prepare-draft
        (mew-draft-find-and-switch draft)
        (mew-delete-directory-recursively attachdir)
@@ -76,35 +68,36 @@ Key assignment:
        (mew-header-prepared)
        (goto-char (point-min))
        (end-of-line)
-       (mew-tinfo-set-hdr-file (mew-expand-folder fld msg))
+       (mew-tinfo-set-hdr-file (mew-expand-msg fld msg))
        (mew-tinfo-set-field-del ,fields)))))
+(put 'mew-summary-header-mode 'lisp-indent-function 1)
 
 (defun mew-summary-send-to-others ()
   "Prepare Header mode so that this message is to be sent
 other people by a header re-write."
   (interactive)
-  (mew-summary-header-mode
-   mew-field-delete-for-others
-   (let ((case (mew-tinfo-get-case)))
-     (mew-draft-header-insert mew-to: "")
-     (mew-draft-header-insert mew-cc: (mew-cc case))
-     (mew-draft-header-insert mew-from: (mew-from case))
-     (mew-draft-header-insert mew-fcc: (mew-fcc case))
-     (mew-draft-header-insert mew-dcc: (mew-dcc case))
-     (mew-draft-header-insert mew-organization: (mew-organization case)))))
+  (mew-summary-header-mode mew-field-delete-for-others
+    (let ((case (mew-tinfo-get-case)))
+      (mew-draft-header-insert mew-to: "")
+      (mew-draft-header-insert mew-cc: (mew-cc case))
+      (mew-draft-header-insert mew-from: (mew-from case))
+      (mew-draft-header-insert mew-fcc: (mew-fcc case))
+      (mew-draft-header-insert mew-bcc: (mew-bcc case))
+      (mew-draft-header-insert mew-dcc: (mew-dcc case))
+      (mew-draft-header-insert mew-organization: (mew-organization case)))))
 
 (defun mew-summary-resend ()
   "Prepare Header mode so that this message is to be forwarded
 by a header re-write."
   (interactive)
-  (mew-summary-header-mode
-   mew-field-delete-for-resending
-   (let ((case (mew-tinfo-get-case)))
-     (mew-draft-header-insert mew-resent-to: "")
-     (mew-draft-header-insert mew-resent-cc: (mew-cc case))
-     (mew-draft-header-insert mew-resent-from: (mew-from case))
-     (mew-draft-header-insert mew-resent-fcc: (mew-fcc case))
-     (mew-draft-header-insert mew-resent-dcc: (mew-dcc case)))))
+  (mew-summary-header-mode mew-field-delete-for-resending
+    (let ((case (mew-tinfo-get-case)))
+      (mew-draft-header-insert mew-resent-to: "")
+      (mew-draft-header-insert mew-resent-cc: (mew-cc case))
+      (mew-draft-header-insert mew-resent-from: (mew-from case))
+      (mew-draft-header-insert mew-resent-fcc: (mew-fcc case))
+      (mew-draft-header-insert mew-resent-bcc: (mew-bcc case))
+      (mew-draft-header-insert mew-resent-dcc: (mew-dcc case)))))
 
 (defun mew-header-process-message (action)
   (if (not (file-readable-p (mew-tinfo-get-hdr-file)))
@@ -139,9 +132,8 @@ by a header re-write."
 	    ;; the end of the header
 	    (save-restriction
 	      (narrow-to-region (point) (point))
-	      (mew-frwlet
-	       mew-cs-text-for-read mew-cs-text-for-write
-	       (mew-insert-file-contents (mew-tinfo-get-hdr-file)))
+	      (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+		(mew-insert-file-contents (mew-tinfo-get-hdr-file)))
 	      (goto-char (point-min))
 	      (mew-header-goto-end)
 	      (mew-header-set mew-header-separator)
@@ -189,9 +181,8 @@ by a header re-write."
 	    ;; the end of the header
 	    (save-restriction
 	      (narrow-to-region (point) (point))
-	      (mew-frwlet
-	       mew-cs-text-for-read mew-cs-text-for-write
-	       (mew-insert-file-contents (mew-tinfo-get-hdr-file)))
+	      (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+		(mew-insert-file-contents (mew-tinfo-get-hdr-file)))
 	      (goto-char (point-min))
 	      (mew-header-goto-end)
 	      (mew-header-set mew-header-separator)
@@ -214,13 +205,13 @@ by a header re-write."
 	      (mew-summary-visit-folder postq)))))))
 
 (defun mew-header-make-message ()
-  "Compose a MIME message from the header and its body 
+  "Compose a MIME message from the header and its body
 then put it into a queue folder."
   (interactive)
   (mew-header-process-message 'queue))
 
 (defun mew-header-send-message ()
-  "Compose a MIME message from the header and its body 
+  "Compose a MIME message from the header and its body
 then send it."
   (interactive)
   (mew-header-process-message 'send))
@@ -231,7 +222,7 @@ then send it."
 ;;;
 
 (defun mew-summary-save (&optional askcs)
-  "Save any parts. If the target is a message, you are asked which 
+  "Save any parts. If the target is a message, you are asked which
 you want to save, the entire message or its body. If the target is
 a non-message part, the part is saved (with line delimiter conversion
 if it is a text object). When executed with '\\[universal-argument]', coding-system for
@@ -240,7 +231,7 @@ a text object to be saved can be specified."
   (mew-summary-msg-or-part
    ;; need to make a cache or a message buffer.
    (mew-summary-toggle-disp-msg 'on)
-   (mew-summary-display nil)
+   (mew-summary-display)
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number2))
 	  (num (mew-syntax-number))
@@ -274,10 +265,10 @@ a text object to be saved can be specified."
 	     (setq body (mew-syntax-get-entry body '(1))))
 	   (setq bct (mew-syntax-get-value (mew-syntax-get-ct body) 'cap))
 	   (if (mew-ct-textp bct)
-	       (if (y-or-n-p (format "Save the entire message (y) or %s (n) " bodyname))
+	       (if (y-or-n-p (format "Save the entire message (y) or %s (n)? " bodyname))
 		   (setq have-hdrp t)
 		 (if plus1p
-		     (setq nums (nreverse (cons 1 (nreverse nums))))
+		     (setq nums (nconc nums '(1)))
 		   (setq bodyp t))
 		 (setq PLUS1P (mew-syntax-get-privacy body)))
 	     (setq have-hdrp t))))
@@ -328,7 +319,7 @@ a text object to be saved can be specified."
 	     (if linebasep
 		 (setq writecs mew-cs-text-for-write)
 	       (setq writecs mew-cs-binary))
-	     (when (and askcs linebasep mew-mule-p)
+	     (when (and askcs linebasep)
 	       (setq tocs (read-coding-system "Coding-system: "))
 	       (if (mew-cs-raw-p tocs)
 		   (setq writecs tocs tocs nil)))
@@ -339,7 +330,7 @@ a text object to be saved can be specified."
 	     (when tocs
 	       (setq charset (mew-syntax-get-param ctl "charset"))
 	       (if charset (setq fromcs (mew-charset-to-cs charset)))
-	       ;; When saving an entire message or its body, 
+	       ;; When saving an entire message or its body,
 	       ;; its charset is unknown since it is not analyzed.
 	       ;; So, we make use of mew-cs-autoconv.
 	       (unless (mew-coding-system-p fromcs)
@@ -349,10 +340,9 @@ a text object to be saved can be specified."
 	       (mew-cs-decode-region
 		(point-min) (point-max) (or fromcs mew-cs-autoconv))
 	       (mew-cs-encode-region (point-min) (point-max) tocs))
-	     (mew-frwlet
-	      mew-cs-dummy writecs
-	      ;; do not specify 'no-msg
-	      (write-region (point-min) (point-max) file append-p)))))))))
+	     (mew-frwlet mew-cs-dummy writecs
+	       ;; do not specify 'no-msg
+	       (write-region (point-min) (point-max) file append-p)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -364,124 +354,111 @@ a text object to be saved can be specified."
 according to a specified language.
 If executed with '\\[universal-argument]', coding-system is asked."
   (interactive "P")
-  (if (not mew-mule-p)
-      (message "This command cannot be used on non-Mule Emacs")
-    (mew-summary-msg-or-part
-     ;; need to make a cache or a message buffer.
-     (mew-summary-toggle-disp-msg 'on)
-     (mew-summary-display nil)
-     (let* ((fld (mew-summary-folder-name))
-	    (msg (mew-summary-message-number2))
-	    (num (mew-syntax-number))
-	    (nums (mew-syntax-number-to-nums num))
-	    (cbuf (mew-cache-hit fld msg))
-	    (alt  (mew-cache-dinfo-get-use-alt cbuf))
-	    (cache (or cbuf (mew-buffer-message)))
-	    (syntax (mew-syntax-get-entry (mew-cache-decode-syntax cache) nums))
-	    (buf (generate-new-buffer mew-buffer-prefix))
-	    (win (selected-window))
-	    (orig-lang current-language-environment)
-	    PLUS1P limit bodyp hbeg hend beg end start
-	    lang prompt tocs cs-env)
-       (save-excursion
-	 ;; First of all, we should determine which part the user want to
-	 ;; save due to the ambiguity.
-	 ;; "y" on Message/Rfc822
-	 ;;     - msg/txt      the entire msg or its body?
-	 ;;     - msg/mul/txt  the entire msg or its part 1?
-	 ;; We have to make use of mew-decode-syntax
-	 ;; in the cache buffer due to the PGP/MIME dilemma.
-	 ;; We need the correct LIMIT.
-	 (when (mew-syntax-message-p syntax)
-	   (let (body plus1p)
-	     (setq body (mew-syntax-get-part syntax))
-	     (when (mew-syntax-multipart-p body)
-	       (setq plus1p t)
-	       (setq body (mew-syntax-get-entry body '(1))))
-	     (if plus1p
-		 (setq nums (nreverse (cons 1 (nreverse nums))))
-	       (setq bodyp t))
-	     (setq PLUS1P (mew-syntax-get-privacy body))))
-	 ;; Now, let's analyze the message in the burst buffer.
-	 ;; This is lengthy, though, avoidable.
-	 (setq limit (length nums))
-	 (if (or PLUS1P bodyp) ;; VERY important for PGP/MIME
-	     (setq limit (1+ limit)))
-	 ;;
-	 (set-buffer buf)
-	 (mew-erase-buffer)
-	 (mew-insert-message fld msg mew-cs-text-for-read nil)
-	 ;;
-	 (cond
-	  (askcs
-	   (setq prompt
-		 (format "Coding-system (autoconv for %s): " orig-lang))
-	   (setq tocs (or (read-coding-system prompt) mew-cs-autoconv)))
-	  (t
-	   (setq tocs mew-cs-autoconv)
-	   (setq prompt (format "Language (%s): " orig-lang))
-	   (setq lang (mew-input-language-name prompt orig-lang))
-	   (setq cs-env (mew-set-language-environment-coding-systems lang))))
-	 (mew-dinfo-set limit 'no-cs-conv t alt)
-	 (mew-decode-for-edit)
-	 ;;
-	 (setq syntax (mew-syntax-get-entry mew-decode-syntax nums))
-	 (when bodyp
-	   (setq hbeg (mew-syntax-get-begin syntax))
-	   (setq hend (mew-syntax-get-end syntax))
-	   (setq syntax (mew-syntax-get-part syntax)))
-	 (setq beg (mew-syntax-get-begin syntax))
-	 (setq end (mew-syntax-get-end syntax))
-	 ;;
-	 (set-buffer (mew-buffer-message))
-	 (select-window (get-buffer-window (current-buffer)))
-	 (widen)
-	 (mew-elet
-	  (delete-region (point-min) (point-max))
-	  (when bodyp
-	    ;; This must be "insert-buffer-substring".
-	    (insert-buffer-substring buf hbeg hend)
-	    (mew-header-arrange (point-min) (point-max)))
-	  (setq start (point))
-	  (save-excursion
-	    (mew-insert-buffer-substring buf beg end)
-	    (mew-cs-decode-region start (point-max) tocs)
-	    (mew-highlight-body-region start (point-max)))
-	  ;; Page breaks
-	  (when mew-break-pages
-	    (goto-char (point-min))
-	    (mew-message-narrow-to-page))
-	  (when (and (not askcs) cs-env)
-	    (mew-reset-coding-systems (car cs-env) (cdr cs-env))))
-	 (mew-summary-display-postscript 'no-hook)
-	 (select-window win))
-       (mew-remove-buffer buf)))))
+  (mew-summary-msg-or-part
+   ;; need to make a cache or a message buffer.
+   (mew-summary-toggle-disp-msg 'on)
+   (mew-summary-display)
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number2))
+	  (num (mew-syntax-number))
+	  (nums (mew-syntax-number-to-nums num))
+	  (cbuf (mew-cache-hit fld msg))
+	  (alt  (mew-cache-dinfo-get-use-alt cbuf))
+	  (cache (or cbuf (mew-buffer-message)))
+	  (syntax (mew-syntax-get-entry (mew-cache-decode-syntax cache) nums))
+	  (buf (generate-new-buffer mew-buffer-prefix))
+	  (win (selected-window))
+	  (orig-lang current-language-environment)
+	  PLUS1P limit bodyp hbeg hend beg end start
+	  lang prompt tocs cs-env)
+     (save-excursion
+       ;; First of all, we should determine which part the user want to
+       ;; save due to the ambiguity.
+       ;; "y" on Message/Rfc822
+       ;;     - msg/txt      the entire msg or its body?
+       ;;     - msg/mul/txt  the entire msg or its part 1?
+       ;; We have to make use of mew-decode-syntax
+       ;; in the cache buffer due to the PGP/MIME dilemma.
+       ;; We need the correct LIMIT.
+       (when (mew-syntax-message-p syntax)
+	 (let (body plus1p)
+	   (setq body (mew-syntax-get-part syntax))
+	   (when (mew-syntax-multipart-p body)
+	     (setq plus1p t)
+	     (setq body (mew-syntax-get-entry body '(1))))
+	   (if plus1p
+	       (setq nums (nconc nums '(1)))
+	     (setq bodyp t))
+	   (setq PLUS1P (mew-syntax-get-privacy body))))
+       ;; Now, let's analyze the message in the burst buffer.
+       ;; This is lengthy, though, avoidable.
+       (setq limit (length nums))
+       (if (or PLUS1P bodyp) ;; VERY important for PGP/MIME
+	   (setq limit (1+ limit)))
+       ;;
+       (set-buffer buf)
+       (mew-erase-buffer)
+       (mew-insert-message fld msg mew-cs-text-for-read nil)
+       ;;
+       (cond
+	(askcs
+	 (setq prompt
+	       (format "Coding-system (autoconv for %s): " orig-lang))
+	 (setq tocs (or (read-coding-system prompt) mew-cs-autoconv)))
+	(t
+	 (setq tocs mew-cs-autoconv)
+	 (setq prompt (format "Language (%s): " orig-lang))
+	 (setq lang (read-language-name nil prompt orig-lang))
+	 (setq cs-env (mew-set-language-environment-coding-systems lang))))
+       (mew-dinfo-set limit 'no-cs-conv t alt)
+       (mew-decode-for-edit)
+       ;;
+       (setq syntax (mew-syntax-get-entry mew-decode-syntax nums))
+       (when bodyp
+	 (setq hbeg (mew-syntax-get-begin syntax))
+	 (setq hend (mew-syntax-get-end syntax))
+	 (setq syntax (mew-syntax-get-part syntax)))
+       (setq beg (mew-syntax-get-begin syntax))
+       (setq end (mew-syntax-get-end syntax))
+       ;;
+       (set-buffer (mew-buffer-message))
+       (select-window (get-buffer-window (current-buffer)))
+       (widen)
+       (mew-elet
+	(delete-region (point-min) (point-max))
+	(when bodyp
+	  ;; This must be "insert-buffer-substring".
+	  (insert-buffer-substring buf hbeg hend)
+	  (mew-header-arrange (point-min) (point-max)))
+	(setq start (point))
+	(save-excursion
+	  (mew-insert-buffer-substring buf beg end)
+	  (mew-cs-decode-region start (point-max) tocs)
+	  (mew-highlight-body-region start (point-max)))
+	;; Page breaks
+	(when mew-break-pages
+	  (goto-char (point-min))
+	  (mew-message-narrow-to-page))
+	(when (and (not askcs) cs-env)
+	  (mew-reset-coding-systems (car cs-env) (cdr cs-env))))
+       (mew-summary-display-postscript 'no-hook)
+       (select-window win))
+     (mew-remove-buffer buf))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Burst
 ;;;
 
-(defun mew-summary-burst-body (fld msg folder &optional num)
-  (let (ret errmsg mstr m)
-    (with-temp-buffer
-      (mew-insert-message fld msg mew-cs-text-for-read nil)
-      (mew-dinfo-set 1 nil nil nil) ;; xxx
-      (mew-decode-for-edit)
-      (cond
-       ((not (mew-syntax-multipart-p (mew-syntax-get-part mew-decode-syntax)))
-	(message "Cannot burst"))
-       ((not (setq mstr (mew-folder-new-message folder 'num-only)))
-	(setq errmsg (format "Error in %s. Nothing was processed" folder)))
-       (t
-	(setq m (mew-summary-burst-part (mew-syntax-get-part mew-decode-syntax)
-					folder (string-to-int mstr) num))
-	(if (= m (string-to-int mstr))
-	    (message "Nothing was processed")
-	  (mew-touch-folder folder)
-	  (setq ret (list mstr (int-to-string (1- m)))))))
-      (if errmsg (error "%s" errmsg))
-      ret)))
+(defvar mew-burst-last-folder nil)
+
+(defun mew-input-burst-folder ()
+  (let (default)
+     (if (and mew-use-burst-folder-history mew-burst-last-folder)
+	 (setq default mew-burst-last-folder)
+       (setq default (mew-inbox-folder))) ;; local folder
+     (setq mew-burst-last-folder (mew-input-local-folder default))
+     mew-burst-last-folder))
 
 (defun mew-summary-burst-part (part folder newmsg &optional num)
   (let (n len nums entry file)
@@ -500,64 +477,117 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	(setq newmsg (mew-summary-burst-part entry folder newmsg)))
        ((string= (mew-syntax-get-value (mew-syntax-get-ct entry) 'cap)
 		 mew-ct-msg)
-	(setq file (mew-expand-folder folder (int-to-string newmsg)))
-	(mew-frwlet
-	 mew-cs-dummy mew-cs-text-for-write
-	 (write-region
-	  (mew-syntax-get-begin entry)
-	  ;; This is RFC 822 message.
-	  ;; So, body is a single text/plain.
-	  (mew-syntax-get-end (mew-syntax-get-part entry))
-	  file nil 'no-msg))
+	(setq file (mew-expand-new-msg folder (number-to-string newmsg)))
+	(mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	  (write-region
+	   (mew-syntax-get-begin entry)
+	   ;; This is RFC 822 message.
+	   ;; So, body is a single text/plain.
+	   (mew-syntax-get-end (mew-syntax-get-part entry))
+	   file nil 'no-msg))
 	(mew-set-file-modes file)
 	(setq newmsg (1+ newmsg))))
       (setq n (1+ n))
       (setq nums (list n)))
     newmsg))
 
-(defvar mew-burst-last-folder nil)
+(defun mew-summary-burst-body (fld msg folder &optional num)
+  (let (ret errmsg mstr m)
+    (with-temp-buffer
+      (mew-insert-message fld msg mew-cs-text-for-read nil)
+      (mew-dinfo-set 1 nil nil nil)
+      (mew-decode-for-edit)
+      (cond
+       ((not (mew-syntax-multipart-p (mew-syntax-get-part mew-decode-syntax)))
+	(message "Cannot burst"))
+       ((not (setq mstr (mew-folder-new-message folder 'num-only)))
+	(setq errmsg (format "Error in %s. Nothing was processed" folder)))
+       (t
+	(setq m (mew-summary-burst-part (mew-syntax-get-part mew-decode-syntax)
+					folder (string-to-number mstr) num))
+	(if (= m (string-to-number mstr))
+	    (message "Nothing was processed")
+	  (mew-touch-folder folder)
+	  (setq ret (list mstr (number-to-string (1- m)))))))
+      (if errmsg (error "%s" errmsg))
+      ret)))
 
-(defun mew-input-burst-folder ()
-  (let (default)
-     (if (and mew-use-burst-folder-history mew-burst-last-folder)
-	 (setq default mew-burst-last-folder)
-       (setq default (mew-inbox-folder))) ;; local folder
-     (setq mew-burst-last-folder (mew-input-local-folder default))
-     mew-burst-last-folder))
+(defun mew-summary-burst-files (fld msg dir)
+  (with-temp-buffer
+    (let (cnt syntax len stx ctl ct)
+      (mew-insert-message fld msg mew-cs-text-for-read nil)
+      ;; must be a message, not a part, so limit is nil
+      (mew-dinfo-set nil 'no-cs-conv t nil)
+      (mew-decode-for-edit)
+      (setq syntax (mew-syntax-get-part mew-decode-syntax))
+      (if (mew-syntax-singlepart-p syntax)
+	  (progn
+	    (mew-syntax-set-cdp syntax (mew-syntax-cdp-format mew-ct-txt mew-draft-coverpage))
+	    (mew-d2e-singlepart syntax dir))
+	(setq cnt mew-syntax-magic)
+	(setq len (length syntax))
+	(while (< cnt len)
+	  (setq stx (aref syntax cnt))
+	  (setq ctl (mew-syntax-get-ct stx))
+	  (setq ct (mew-syntax-get-value ctl 'cap))
+	  (cond
+	   ((string= mew-ct-msg ct)
+	    (mew-d2e-message stx dir))
+	   ((mew-ct-multipartp ct)
+	    (mew-d2e-multipart stx (mew-random-filename dir 1 nil)))
+	   (t
+	    (if (= cnt mew-syntax-magic)
+		(mew-syntax-set-cdp stx (mew-syntax-cdp-format mew-ct-txt mew-draft-coverpage)))
+	    (mew-d2e-singlepart stx dir)))
+	  (setq cnt (1+ cnt)))))))
 
 (defun mew-summary-burst (&optional arg)
-  "De-capsulate messages embedded in this message.
-If called with '\\[universal-argument]', the current part is de-capsulated."
+  "Decompose messages embedded in this message.
+You are asked a folder for the decomposition.
+
+If executed on a part, only the message part is saved to
+a folder which you specify.
+
+If called with '\\[universal-argument]', any files in this message
+are decomposed. You are asked a new directory name for the decomposition."
   (interactive "P")
   (mew-summary-msg-or-part
    (let ((fld (mew-summary-folder-name))
 	 (msg (mew-summary-message-number2))
-	 (folder (mew-input-burst-folder))
-	 (num (if arg (mew-syntax-number)))
-	 ret)
-     (when (and folder (mew-local-folder-check folder 'ask))
+	 (num (mew-syntax-number))
+	 folder dir ret)
+     (cond
+      (arg ;; file decomposition
+       (setq dir (read-file-name "New directory: " mew-home))
+       (while (file-exists-p dir)
+	 (setq dir (read-file-name (format "\"%s\" exists. New directory: " dir) mew-home)))
+       (mew-make-directory dir)
        (message "Bursting...")
-       (setq ret (mew-summary-burst-body fld msg folder num))
-       (when ret
-	 (message "Bursting...done")
-	 (when mew-ask-folder-after-burst
-	   (if (y-or-n-p (format "Go to %s? " folder))
-	       (mew-summary-visit-folder folder 'goend)))
-	 (message "Messages from %s to %s were extracted in %s"
-		  (nth 0 ret) (nth 1 ret) folder))))))
+       (mew-summary-burst-files fld msg dir)
+       (message "Bursting...done"))
+      (t
+       (setq folder (mew-input-burst-folder))
+       (when (and folder (mew-local-folder-check folder 'ask))
+	 (message "Bursting...")
+	 (setq ret (mew-summary-burst-body fld msg folder num))
+	 (when ret
+	   (message "Bursting...done")
+	   (when mew-ask-folder-after-burst
+	     (if (y-or-n-p (format "Go to %s? " folder))
+		 (mew-summary-visit-folder folder 'goend)))
+	   (message "Messages from %s to %s were extracted in %s"
+		    (nth 0 ret) (nth 1 ret) folder))))))))
 
 (defun mew-summary-burst-multi ()
-  "De-capsulate messages embedded in the messages marked with '*'."
+  "Decompose messages embedded in the messages marked with '*'."
   (interactive)
   (mew-summary-multi-msgs
    (let ((folder (mew-input-burst-folder))
 	 (targets FLD-MSG-LIST))
      (when (and folder (mew-local-folder-check folder 'ask))
        (message "Bursting...")
-       (while targets
-	 (mew-summary-burst-body
-	  (car (car targets)) (cdr (car targets)) folder)
-	 (setq targets (cdr targets)))
+       (dolist (target targets)
+	 (mew-summary-burst-body (car target) (cdr target) folder))
        (message "Bursting...done")
        (when mew-ask-folder-after-burst
 	 (if (y-or-n-p (format "Go to %s? " folder))
@@ -587,7 +617,7 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	 (forward-line 2)
 	 (setq start (point))
 	 (setq mstr (mew-folder-new-message folder 'num-only))
-	 (setq m (string-to-int mstr))
+	 (setq m (string-to-number mstr))
 	 (while (re-search-forward sep2 nil t)
 	   (save-excursion
 	     (beginning-of-line)
@@ -596,15 +626,14 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	       (goto-char (point-min))
 	       (while (re-search-forward "^- " nil t)
 		 (replace-match "" nil t))
-	       (mew-frwlet
-		mew-cs-dummy mew-cs-text-for-write
-		(write-region (point-min) (point-max)
-			      (mew-expand-folder folder mstr)
-			      nil 'no-msg))))
+	       (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+		 (write-region (point-min) (point-max)
+			       (mew-expand-new-msg folder mstr)
+			       nil 'no-msg))))
 	   (forward-line 2)
 	   (setq start (point))
 	   (setq m (1+ m))
-	   (setq mstr (int-to-string m)))
+	   (setq mstr (number-to-string m)))
 	 (message "Bursting...done")
 	 (mew-touch-folder folder)
 	 (setq done t)))
@@ -623,7 +652,7 @@ typically included in a MIME-encapsulated error message.
 
 1. In +draft, the message is just edited.
 2. In +queue or +postq, the message is moved to +draft and is edited.
-3. Otherwise, the message is copied to +draft and is edited. 
+3. Otherwise, the message is copied to +draft and is edited.
 
 For the other folders, if mew-case-guess-when-prepared is 't', each
 fields of the original header is replaced according to a guessed
@@ -639,7 +668,7 @@ See also mew-summary-edit-again."
 	 draftname)
      (cond
       ((string= fld mew-draft-folder)
-       (setq draftname (mew-path-to-folder (mew-expand-folder fld msg)))
+       (setq draftname (mew-path-to-folder (mew-expand-msg fld msg)))
        (if (get-buffer draftname)
 	   (progn
 	     (mew-current-set-window-config) ;; xxx necessary?
@@ -669,11 +698,12 @@ See also mew-summary-edit-again."
 ;;; Header
 ;;;
 
-(defsubst mew-summary-edit-header-common ()
+(defun mew-summary-edit-header-common ()
   (mew-header-delete-lines (list mew-x-mailer:))
   (mew-header-goto-end)
   ;; X-Mailer: must be the last
-  (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
+  (if (mew-use-x-mailer (mew-tinfo-get-case))
+      (mew-draft-header-insert mew-x-mailer: mew-x-mailer))
   (mew-header-clear) ;; erase the old header separator
   (mew-header-set "\n"))
 
@@ -718,7 +748,7 @@ Note that the message is not a valid MIME message."
   (mew-summary-prepare-draft
    ;; mew-cs-m17n
    ;; case is copied here
-   (mew-draft-find-and-switch (mew-expand-folder fld msg))
+   (mew-draft-find-and-switch (mew-expand-new-msg fld msg))
    (mew-summary-edit-header-for-draft)
    (mew-draft-mode)
    (mew-draft-rehighlight-body)
@@ -742,7 +772,7 @@ The message is assumed to be a valid MIME message."
   ;; main part
   (mew-summary-prepare-draft
    (mew-summary-edit-message fld msg nil nil)
-   (let* ((file (mew-expand-folder fld msg))
+   (let* ((file (mew-expand-msg fld msg))
 	  (work (concat file mew-queue-work-suffix))
 	  (info (concat file mew-queue-info-suffix))
 	  inf hdr)
@@ -794,7 +824,7 @@ The message is assumed to be a valid MIME message."
 (defun mew-summary-edit-again ()
   "Edit an old fashioned error message in which the original message
 is encapsulated after strings defined in 'mew-summary-edit-again-regex'
-An example is \"----- Original message follows -----\". See also 
+An example is \"----- Original message follows -----\". See also
 mew-summary-reedit."
   (interactive)
   (mew-summary-msg
@@ -815,10 +845,9 @@ mew-summary-reedit."
 	 ;; skip blank lines
 	 (while (looking-at "^$") (forward-line))
 	 (delete-region (point-min) (point))
-	 (setq nfile (mew-expand-folder nfld nmsg))
-	 (mew-frwlet
-	  mew-cs-dummy mew-cs-text-for-write
-	  (write-region (point-min) (point-max) nfile nil 'no-msg))
+	 (setq nfile (mew-expand-new-msg nfld nmsg))
+	 (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	   (write-region (point-min) (point-max) nfile nil 'no-msg))
 	 ;; A new message is stored in the temporary directory
 	 (mew-summary-reedit-for-message nfld nmsg nil nil nil)
 	 (run-hooks 'mew-draft-mode-edit-again-hook)
@@ -833,18 +862,17 @@ mew-summary-reedit."
   "Get an appropriate suffix from content type."
   (let* ((base (mew-ctdb-regex (mew-ctdb-by-ct ct)))
 	 (len (length base))
-	 (ret (make-string len ?a))
-	 (i 0) (j 0) char)
+	 (ret (mew-make-string len))
+	 (j 0) char)
     (if (not (stringp base))
 	nil
-      (while (< i len)
+      (dotimes (i len)
 	(setq char (aref base i))
 	(if (or (char-equal char ?.)
 		(and (>= char ?a) (<= char ?z)))
 	    (progn
 	      (aset ret j char)
-	      (setq j (1+ j))))
-	(setq i (1+ i)))
+	      (setq j (1+ j)))))
       (setq ret (substring ret 0 j))
       (if (string-match "^\\.[a-z]+$" ret)
 	  ret
@@ -860,7 +888,7 @@ mew-summary-reedit."
     (mew-draft-find-and-switch draft)
     (mew-delete-directory-recursively attachdir)
     ;; De-compose the message in the burst buffer.
-    ;; do not decode Message/Rfc822 parts. 
+    ;; do not decode Message/Rfc822 parts.
     ;; They will be just a attachment message.
     (save-excursion
       ;; Let's decode the message without limitations
@@ -888,15 +916,15 @@ mew-summary-reedit."
 	(unless (string= mew-ct-txt
 			 (mew-syntax-get-value
 			  (mew-syntax-get-ct (aref syntax cnt)) 'cap))
-	  (setq syntax(mew-syntax-insert-entry
-		       syntax '(1) (mew-encode-syntax-text))))
+	  (setq syntax (mew-syntax-insert-entry
+			syntax '(1) (mew-encode-syntax-text))))
 	(setq len (length syntax))
 	(mew-make-directory attachdir)
 	(while (< cnt len)
 	  (setq stx (aref syntax cnt))
 	  (setq ctl (mew-syntax-get-ct stx))
 	  (setq ct (mew-syntax-get-value ctl 'cap))
-	  (cond 
+	  (cond
 	   ((string= mew-ct-msg ct)
 	    (mew-d2e-message stx attachdir))
 	   ((mew-ct-multipartp ct)
@@ -951,10 +979,8 @@ mew-summary-reedit."
 	(setq file (expand-file-name file dir))
       (setq file (mew-random-filename dir 2 nil (mew-ct-suffix ct)))
       (mew-syntax-set-cdp syntax nil))
-    (mew-frwlet
-     mew-cs-dummy
-     (if linebasep mew-cs-text-for-write mew-cs-binary)
-     (write-region beg end file nil 'no-msg))
+    (mew-frwlet mew-cs-dummy (if linebasep mew-cs-text-for-write mew-cs-binary)
+      (write-region beg end file nil 'no-msg))
     (if (and (mew-ct-textp ct) (mew-syntax-get-param ctl "charset"))
 	(mew-delete "charset" ctl)) ;; side effect
     (mew-syntax-set-file syntax (file-name-nondirectory file))
@@ -971,7 +997,7 @@ mew-summary-reedit."
     (while (< cnt len)
       (setq part (aref syntax cnt))
       (setq ct (mew-syntax-get-value (mew-syntax-get-ct part) 'cap))
-      (cond 
+      (cond
        ((string= mew-ct-msg ct)
 	(mew-d2e-message part dir))
        ((mew-ct-multipartp ct)
@@ -995,9 +1021,8 @@ mew-summary-reedit."
   (let ((file (mew-random-filename dir 2 t))
 	(beg (mew-syntax-get-begin syntax))
 	(end (mew-syntax-get-end (mew-syntax-get-part syntax))))
-    (mew-frwlet
-     mew-cs-dummy mew-cs-text-for-write
-     (write-region beg end file nil 'no-msg))
+    (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+      (write-region beg end file nil 'no-msg))
     (mew-syntax-set-key syntax 'single)
     (mew-syntax-set-file syntax (file-name-nondirectory file))
     (mew-syntax-set-decrypters syntax nil)
@@ -1010,13 +1035,13 @@ mew-summary-reedit."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2005 Mew developing team.
+;; Copyright (C) 1999-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1025,7 +1050,7 @@ mew-summary-reedit."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
