@@ -56,11 +56,8 @@
 
 (defun mew-toolbar-make (map alist)
   (let ((tool-bar-map (make-sparse-keymap)) ;; for tool-bar-add-item-from-menu
-	(data-directory mew-icon-directory)
-	a)
-    (while alist
-      (setq a (car alist))
-      (setq alist (cdr alist))
+	(data-directory mew-icon-directory))
+    (dolist (a alist)
       (if (fboundp 'tool-bar-local-item-from-menu)
 	  ;; Emacs 21.3.50 or later
 	  (tool-bar-local-item-from-menu (car a) (cdr a) tool-bar-map map)
@@ -89,23 +86,23 @@
       (set (make-local-variable 'tool-bar-map)
 	   (mew-toolbar-make mew-header-mode-map mew-header-toolbar-spec))))
 
-(defsubst mew-summary-toolbar-update ())
-(defsubst mew-message-toolbar-update ())
-(defsubst mew-draft-toolbar-update ())
-(defsubst mew-header-toolbar-update ())
+(defun mew-summary-toolbar-update ())
+(defun mew-message-toolbar-update ())
+(defun mew-draft-toolbar-update ())
+(defun mew-header-toolbar-update ())
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; End of messages
 ;;;
 
-(defsubst mew-message-set-end-of-message ()
+(defun mew-message-set-end-of-message ()
   (overlay-put (mew-minfo-get-eom) 'before-string mew-end-of-message-string))
 
-(defsubst mew-message-set-end-of-part ()
+(defun mew-message-set-end-of-part ()
   (overlay-put (mew-minfo-get-eom) 'before-string mew-end-of-part-string))
 
-(defsubst mew-message-set-end-of-nil ()
+(defun mew-message-set-end-of-nil ()
   (overlay-put (mew-minfo-get-eom) 'before-string nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -134,95 +131,181 @@
 ;;; Image
 ;;;
 
+(defcustom mew-image-display-resize t
+  "*If non-nil, image will be displayed with fitting to frame size"
+  :group 'mew-message
+  :type 'boolean)
+
+(defcustom mew-image-display-resize-care-height t
+  "*If nil, image will be displayed with fitting to only frame width"
+  :group 'mew-message
+  :type 'boolean)
+
+(defcustom mew-image-width-margin 45
+  "*A value for width margin to display image when resizing"
+  :group 'mew-message
+  :type 'integer)
+
+(defcustom mew-image-height-margin 200
+  "A value for height margin to display image when resizing"
+  :group 'mew-message
+  :type 'integer)
+
+(defvar mew-image-l-endian ?\x49)
+(defvar mew-image-b-endian ?\x4d)
+
 (defun mew-image-inline-p (format)
   ;; display-graphic-p
   (and window-system (image-type-available-p format)))
 
-(defun mew-jpeg-width ()
-  (let (c size width)
+(defun mew-img-get-n (op len)
+  (let* ((size 0))
+    (if (eq op ?\x49)   ; I(?\x49) or M(?\x4d)
+	(dotimes (n len)
+	  (setq size (+ size (* (char-after) (expt ?\x100 n))))
+	  (forward-char))
+      (while (< 0 len)
+	  (setq len (- len 1))
+	  (setq size (+ size (* (char-after) (expt ?\x100 len))))
+	  (forward-char)))
+    size))
+
+(defun mew-jpeg-size ()
+  (let (c size width height)
     (save-excursion
       (catch 'loop
 	(while t
 	  (setq c (char-after))
 	  (forward-char)
-	  (unless (eq c #xff)
+	  (unless (eq c ?\xff)
 	    (throw 'loop nil))
 	  (setq c (char-after))
 	  (forward-char)
 	  (cond
-	   ((eq c #xd8)
-	    )
-	   ((eq c #xe0)
-	    (forward-char 16))
-	   ((or (eq c #xdb) (eq c #xc4)
-		(and (>= c #xe0) (<= c #xed)))
-	    (setq size (char-after))
-	    (forward-char)
-	    (setq size (+ (* size 256) (char-after)))
-	    (forward-char)
-	    (forward-char (- size 2)))
-	   ((eq c #xc0)
-	    (forward-char 5)
-	    (setq width (char-after))
-	    (forward-char)
-	    (setq width (+ (* width 256) (char-after)))
+	   ((or (eq c ?\xd8)                      ;; SOI
+		(eq c ?\xd9)                      ;; EOI
+		(and (>= c ?\xd0) (<= c ?\xd7)))) ;; RSTm
+	   ((eq c ?\xc0)
+	    (forward-char 3)
+	    (setq height (mew-img-get-n mew-image-b-endian 2))
+	    (setq width (mew-img-get-n mew-image-b-endian 2))
+	    (backward-char)
 	    (throw 'loop nil))
+	   ((and (>= c ?\xc1) (<= c ?\xfe))
+	    (setq size (mew-img-get-n mew-image-b-endian 2))
+	    (forward-char (- size 2)))
 	   (t
 	    (throw 'loop nil))))))
-    width))
+    (cons width height)))
 
-(defun mew-png-width ()
-  (let (width)
+(defun mew-png-size ()
+  (let (width height)
     (save-excursion
       (forward-char 18)
       ;; length is four bytes
       ;; but we takes lower two bytes
-      (setq width (char-after))
-      (forward-char)
-      (setq width (+ (* width 256) (char-after)))
-      width)))
+      (setq width (mew-img-get-n mew-image-b-endian 2))
+      (forward-char 2)
+      (setq height (mew-img-get-n mew-image-b-endian 2))
+      (cons width height))))
 
-(defvar mew-image-width-margin 45)
+(defun mew-gif-size ()
+  (let (width height)
+    (save-excursion
+      (forward-char 6)
+      (setq width (mew-img-get-n mew-image-l-endian 2))
+      (setq height (mew-img-get-n mew-image-l-endian 2))
+      (cons width height))))
+
+(defun mew-tiff-size ()
+  (let (endian size entry tag width height)
+    (save-excursion
+      (setq endian (char-after))
+      (forward-char 4)
+      (setq size (mew-img-get-n endian 4))
+      (forward-char (- size 8)) ;; jump to IFD
+      ;; IFD
+      (setq entry (mew-img-get-n endian 2))
+      (catch 'loop
+	(while t
+	  (setq entry (- entry 1))
+	  (setq tag (mew-img-get-n endian 2))
+	  (cond
+	   ((eq tag 256)
+	    (forward-char 6)
+	    (setq width (mew-img-get-n endian 2))
+	    (forward-char 2))
+	   ((eq tag 257)
+	    (forward-char 6)
+	    (setq height (mew-img-get-n endian 2))
+	    (forward-char 2))
+	   (t
+	    (forward-char 10)))
+	  (unless (< 1 entry)
+	    (throw 'loop nil)))))
+    (cons width height)))
 
 (defvar mew-image-alist
-  '((jpeg "jpegtopnm" mew-jpeg-width)
-    (png  "pngtopnm"  mew-png-width)))
+  '((jpeg mew-jpeg-size "jpegtopnm" "pnmtojpeg")
+    (png  mew-png-size  "pngtopnm"  "pnmtopng")
+    (gif  mew-gif-size  "giftopnm"  "pnmtogif")
+    (tiff mew-tiff-size "tifftopnm" "pnmtotiff")))
 
 (defun mew-image-format-ent (format)
   (assoc format mew-image-alist))
 
-(defun mew-image-get-prog (ent)
+(defun mew-image-get-func (ent)
   (nth 1 ent))
 
-(defun mew-image-get-func (ent)
+(defun mew-image-get-prog (ent)
   (nth 2 ent))
 
+(defun mew-image-get-prog2 (ent)
+  (nth 3 ent))
+
 (defun mew-mime-image (cache begin end format)
   (message "Loading image...")
   (set-buffer (mew-buffer-message))
   (let* ((width (- (frame-pixel-width (selected-frame)) mew-image-width-margin))
+	 (height (- (frame-pixel-height (selected-frame)) mew-image-height-margin))
 	 (ent (mew-image-format-ent format))
 	 (prog (mew-image-get-prog ent))
-	 (func (mew-image-get-func ent))
-	 image-width image)
-    (when (and func (fboundp func))
+	 (prog2 (mew-image-get-prog2 ent))
+	 (func-size (mew-image-get-func ent))
+	 image-width image-height image)
+    (when (and mew-image-display-resize func-size (fboundp func-size))
       (save-excursion
 	(set-buffer cache)
 	(goto-char begin)
-	(setq image-width (funcall func))))
-    (if (and image-width (< width image-width) (mew-which-exec prog))
+	(setq image-width (car (funcall func-size)))
+	(setq image-height (cdr (funcall func-size)))))
+    (if (and image-width image-height
+	     (or (< width image-width)
+		 (and mew-image-display-resize-care-height (< height image-height)))
+	     (mew-which-exec prog))
 	(with-temp-buffer
-	  (message "Resizing image...")
-	  (insert-buffer-substring cache begin end)
-	  (mew-set-buffer-multibyte nil)
-	  (call-process-region (point-min) (point-max) prog
-			       t '(t nil) nil)
-	  (call-process-region (point-min) (point-max) "pnmscale"
-			       t '(t nil) nil
-			       "-xsize" (format "%d" width))
-	  (setq format 'pbm)
-	  (setq image (mew-buffer-substring (point-min) (point-max)))
-	  (message "Resizing image...done"))
+	  (mew-plet
+	   (message "Resizing image...")
+	   (insert-buffer-substring cache begin end)
+	   (mew-set-buffer-multibyte nil)
+	   (call-process-region (point-min) (point-max) prog
+				t '(t nil) nil)
+	   (if mew-image-display-resize-care-height
+	       (call-process-region (point-min) (point-max) "pnmscale"
+				    t '(t nil) nil
+				    "-xysize"
+				    (format "%d" width)
+				    (format "%d" height))
+	     (call-process-region (point-min) (point-max) "pnmscale"
+				  t '(t nil) nil
+				  "-xsize" (format "%d" width)))
+	   (if (and (string< emacs-version "22") ;; xxx
+		    (mew-which-exec prog2))
+	       (call-process-region (point-min) (point-max) prog2
+				    t '(t nil) nil)
+	     (setq format 'pbm))
+	   (setq image (mew-buffer-substring (point-min) (point-max)))
+	   (message "Resizing image...done")))
       (save-excursion
 	(set-buffer cache)
 	(setq image (mew-buffer-substring begin end))))
@@ -259,22 +342,6 @@
 	(goto-char (match-end 1))
 	(insert-image xface)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Profile
-;;;
-
-(defun mew-profile-init ()
-  (when (and mew-profile mew-profile-functions-list)
-    (require 'profile)
-    (let ((profile-functions-list mew-profile-functions-list))
-      (profile-functions))))
-
-(defun mew-profile-results ()
-  (if (and mew-profile mew-profile-functions-list)
-      (let ((profile-buffer mew-buffer-debug))
-	(profile-results))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; SSL/SSH/TLS notification
@@ -282,7 +349,7 @@
 
 (defvar mew-secure-format nil)
 
-;; Emacs 21.x has a bug that an image cannot be displayed 
+;; Emacs 21.x has a bug that an image cannot be displayed
 ;; if it is specified in mode-line-process.
 
 (defvar mew-secure-format2
@@ -297,26 +364,23 @@
 ;;; Click hack
 ;;;
 
-(defun mew-browse-url-at-mouse (event)
-  (interactive "e")
-  (let ((buf (current-buffer)))
-    (browse-url-at-mouse event)
-    (pop-to-buffer buf)))
-
 (define-key mew-message-mode-map [mouse-2] 'mew-browse-url-at-mouse)
-(define-key mew-message-mode-map [M-down-mouse-1] 'mew-browse-url-at-mouse) ;; for Mac
+(if (and (featurep 'emacs) ;; preventing XEmacs's byte compiler from hanging
+	 (eq system-type 'darwin))
+    ;; for Mac
+    (define-key mew-message-mode-map [M-down-mouse-1] 'mew-browse-url-at-mouse))
 
 (provide 'mew-gemacs)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2006 Mew developing team.
+;; Copyright (C) 1997-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -325,7 +389,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
