@@ -38,8 +38,13 @@
 		(save-excursion (beginning-of-line) (point)))))
 
 (defun mew-unhighlight-cursor-line ()
-  (if (overlayp (mew-sinfo-get-cursor-line))
-      (move-overlay (mew-sinfo-get-cursor-line) 1 1)))
+  (when (overlayp (mew-sinfo-get-cursor-line))
+    (move-overlay (mew-sinfo-get-cursor-line) 1 1)
+    ;; (goto-char (point-max)) works wrong if underline exists.
+    ;; Redisplays it so that it works well.
+    ;; (sit-for 0)
+    ;; This code rocks Summary mode when typing "x", sigh.
+    ))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -56,7 +61,7 @@
        'face face))))
 
 (defun mew-highlight-unmark-line ()
-  (remove-text-properties 
+  (remove-text-properties
    (save-excursion (beginning-of-line) (point))
    (save-excursion (end-of-line) (point))
    '(face nil)))
@@ -93,10 +98,8 @@
 	       (setq med (match-end 0))
 	       (forward-line)
 	       (setq nspec (mew-nspec-by-key key))
-	       (setq key-face (or (mew-nspec-keyface nspec)
-				  'mew-face-header-key))
-	       (setq val-face (or (mew-nspec-valface nspec)
-				  'mew-face-header-marginal))
+	       (setq key-face (mew-key-face key nspec))
+	       (setq val-face (mew-val-face key nspec))
 	       (setq overlay (mew-overlay-make beg med))
 	       (overlay-put overlay 'face key-face)
 	       (setq overlay (mew-overlay-make med (1- (point))))
@@ -160,7 +163,7 @@
 	   (url-regex mew-regex-url)
 	   (fancy-num 0)
 	   (fancy-length (length mew-highlight-body-cite-faces))
-	   beg end face fancy-alst fancy-prefix)
+	   beg end face fancy-alst fancy-prefix idn url)
       (save-excursion
 	(mew-elet
 	 (when draft
@@ -195,11 +198,23 @@
 	 (when mew-use-highlight-url
 	   (goto-char BEG)
 	   (while (and (<= (point) END) (re-search-forward url-regex END t))
-	     (setq beg (match-beginning 0))
-	     (setq end (match-end 0))
-	     (put-text-property beg end 'face 'mew-face-body-url)
-	     (put-text-property
-	      beg end 'mouse-face mew-highlight-url-mouse-face))))))))
+	     (save-restriction
+	       (setq beg (match-beginning 0))
+	       (setq end (match-end 0))
+	       (narrow-to-region beg end)
+	       (goto-char (point-min))
+	       (setq url (mew-buffer-substring (point-min) (point-max)))
+	       (while (and mew-use-punycode (re-search-forward mew-regex-punycode nil t))
+		 (setq idn (mew-puny-decode (mew-buffer-substring (match-beginning 0) (match-end 0))))
+		 (when idn
+		   (delete-region (match-beginning 0) (match-end 0))
+		   (insert idn)))
+	       (goto-char (point-max))
+	       (put-text-property (point-min) (point-max) 'mew-url url)
+	       (put-text-property (point-min) (point-max) 'face 'mew-face-body-url)
+	       (put-text-property
+		(point-min) (point-max) 'mouse-face mew-highlight-url-mouse-face)
+	       (setq END (- END (- end (point-max))))))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -213,8 +228,7 @@
 	   (fboundp mew-highlight-x-face-function))
       (funcall mew-highlight-x-face-function beg end)))
 
-(defvar mew-highlight-x-face-function 
-  (if mew-icon-p 'mew-highlight-x-face-original) ;; Emacs 21, XEmacs
+(defvar mew-highlight-x-face-function (if mew-icon-p 'mew-highlight-x-face-original)
   "*A function to display-Xface")
 
 (defun mew-highlight-x-face-original (beg end)
@@ -290,7 +304,6 @@
 		 (throw 'loop (setq ret t)))
 	     (setq med (match-beginning 0))
 	     (forward-line)
-	     (mew-front-nonsticky start med) ;; for XEmacs
 	     (if (and mew-use-highlight-mouse-line window-system)
 		 (put-text-property
 		  start med 'mouse-face mew-highlight-mouse-line-face))
@@ -318,16 +331,13 @@
       ret)))
 
 (defun mew-summary-cook-folders ()
-  (let ((bufs mew-buffers) buf)
-    (save-excursion
-      (while bufs
-	(setq buf (car bufs))
-	(setq bufs (cdr bufs))
-	(when (and (get-buffer buf) (not (input-pending-p)))
-	  (set-buffer buf)
-	  (setq mew-summary-buffer-raw
-		(mew-summary-cook-region
-		 (point-min) (point-max) 'interrupt)))))))
+  (save-excursion
+    (dolist (buf mew-buffers)
+      (when (and (get-buffer buf) (not (input-pending-p)))
+	(set-buffer buf)
+	(setq mew-summary-buffer-raw
+	      (mew-summary-cook-region
+	       (point-min) (point-max) 'interrupt))))))
 
 (defvar mew-highlight-timer-id nil)
 
@@ -345,13 +355,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2006 Mew developing team.
+;; Copyright (C) 1997-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -360,7 +370,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
