@@ -12,7 +12,6 @@
 ;;; Variables
 ;;;
 
-(defvar mew-summary-inbox-position (make-marker))
 (defvar mew-last-shell-command "")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -20,34 +19,32 @@
 ;;; Switching to folder
 ;;;
 
-(defun mew-summary-switch-to-folder (case:folder &optional p-folder func)
-  (let ((ofolder (mew-summary-folder-name 'ext))
-	new-folder)
+(defun mew-summary-switch-to-folder (case:folder &optional func)
+  (let ((ofolder (mew-summary-folder-name 'ext)))
     (cond
      ((get-buffer case:folder)
       (switch-to-buffer case:folder)
       (mew-set-buffer-cs mew-cs-m17n)
-      (when (and p-folder (mew-folder-virtualp case:folder))
-	(mew-vinfo-set-parent-folder p-folder))
       (unless (mew-folder-virtualp case:folder) (mew-summary-folder-cache-load))
-      (unless (string= ofolder case:folder) (mew-window-configure 'summary)))
+      (unless (string= ofolder case:folder) (mew-window-configure 'summary))
+      nil) ;; existing folder
      (t
-      (setq new-folder t)
-      (unless func (setq func 'switch-to-buffer))
-      (funcall func (get-buffer-create case:folder))
+      (funcall (or func 'switch-to-buffer) (get-buffer-create case:folder))
       (mew-set-buffer-cs mew-cs-m17n)
       (mew-buffers-setup case:folder)
-      (mew-sinfo-set-case (mew-case:folder-case case:folder))
-      (mew-sinfo-set-folder (mew-case:folder-folder case:folder))
+      (let* ((case (mew-case:folder-case case:folder))
+	     (folder (mew-case:folder-folder case:folder))
+	     (inboxp (string= folder (mew-proto-inbox-folder folder (or case mew-case)))))
+	(mew-sinfo-set-case case)
+	(mew-sinfo-set-folder folder)
+	(mew-sinfo-set-inboxp inboxp))
       (if (mew-folder-virtualp case:folder)
-	  (progn
-	    (if p-folder (mew-vinfo-set-parent-folder p-folder))
-	    (mew-virtual-mode))
+	  (mew-virtual-mode)
 	(mew-summary-mode)
 	(if mew-summary-trace-directory (cd (mew-expand-folder case:folder)))
 	(mew-summary-folder-cache-load))
-      (mew-window-configure 'summary)))
-    new-folder))
+      (mew-window-configure 'summary)
+      t)))) ;; new-folder
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -60,12 +57,10 @@
 Summary mode."
   (interactive "P")
   (let* ((proto (mew-proto-to-go (mew-summary-folder-name 'ext)))
-	 (case1 (if (mew-folder-remotep proto)
-		    (mew-sinfo-get-case) ;; a remote folder
-		  mew-case-input)) ;; a local/virtual folder
+	 (case1 (car (mew-summary-case-proto)))
 	 (inbox (mew-proto-inbox-folder	proto case1))
 	 (case:folder (mew-input-folder case1 inbox))
-	 case folder buf win frame folder-alist sep)
+	 case folder buf win frame folder-alist)
     (if (null case:folder)
 	(message "The folder does not exist")
       (setq case (mew-case:folder-case case:folder))
@@ -73,14 +68,16 @@ Summary mode."
       (cond
        ((mew-folder-imapp folder)
 	(setq folder-alist (mew-imap-folder-alist case))
-	(setq sep (mew-imap-separator case))
+	;;
 	(unless (or (assoc folder folder-alist)
-		    (assoc (concat folder sep) folder-alist))
-	  (message "%s does not exist" case:folder)
+		    (assoc (concat folder (mew-imap-separator case)) folder-alist))
+	  (message "\"%s\" does not exist. %s" case:folder
+		   (mew-substitute-for-summary "Type '\\[universal-argument]\\[mew-status-update]' to collect IMAP folders"))
 	  (setq case:folder nil)))
        ((mew-folder-nntpp folder)
 	(unless (assoc folder (mew-nntp-folder-alist case))
-	  (message "%s does not exist" case:folder)
+	  (message "\"%s\" does not exist. %s" case:folder
+		   (mew-substitute-for-summary "Type '\\[universal-argument]\\[mew-status-update]' to collect newsgroups"))
 	  (setq case:folder nil))))
       (when case:folder
 	(when mew-use-other-frame-for-summary
@@ -91,11 +88,7 @@ Summary mode."
 		    (raise-frame frame)
 		    (select-frame frame)
 		    ;; Ensure, if possible, that frame gets input focus.
-		    (if (eq window-system 'w32)
-			(w32-focus-frame frame)
-		      (when focus-follows-mouse
-			(set-mouse-position
-			 (selected-frame) (1- (frame-width)) 0))))
+		    (mew-focus-frame frame))
 		(select-frame (make-frame)))
 	    (select-frame (make-frame))))
 	(mew-summary-visit-folder case:folder goend)))))
@@ -141,16 +134,38 @@ Summary mode."
 ;;; Going to a line
 ;;;
 
-(defun mew-summary-goto-line ()
+(defun mew-summary-goto-line (&optional arg)
   "Jump to a line according to the number which you input.
 If 'mew-summary-goto-line-then-display' is non-nil,
 the message is then displayed."
+  (interactive "P")
+  (if arg
+      (let ((msg (read-string "Message No.: " "")))
+	(while (not (string-match mew-regex-message-files3 msg))
+	  (setq msg (read-string "Message No.: " "")))
+	(when (mew-summary-search-msg msg)
+	  (if mew-summary-goto-line-then-display
+	      (mew-summary-display))))
+    (let ((line (string-to-number (read-string "Line No.: " ""))))
+      (when (/= line 0)
+	(goto-line line)
+	(if mew-summary-goto-line-then-display
+	    (mew-summary-display))))))
+
+(defun mew-summary-goto-original-message ()
+  "Go to the original message in the original Summary mode"
   (interactive)
-  (let ((line (string-to-int (read-string "Line No.: " ""))))
-    (when (/= line 0)
-      (goto-line line)
-      (if mew-summary-goto-line-then-display
-	  (mew-summary-display nil)))))
+  (cond
+   ((mew-summary-p)
+    (message "This is the original"))
+   ((mew-virtual-p)
+    (let ((folder (mew-folder-path-to-folder (mew-summary-folder-name) 'has-proto))
+	  (msg (mew-summary-message-number)))
+      (when folder
+	(mew-summary-visit-folder folder nil 'no-ls)
+	(if (mew-summary-search-msg msg)
+	    (mew-summary-display)
+	  (message "No original message found")))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -174,15 +189,16 @@ the message is then displayed."
 ;;;
 
 (defun mew-max-draft-buffer ()
-  (let* ((regex (mew-folder-regex (file-name-as-directory mew-draft-folder)))
-	 (bufs (mew-buffer-list regex))
-	 buf draft)
-    (while bufs
-      (setq buf (car bufs))
-      (setq bufs (cdr bufs))
-      (if (or (null draft) (string< draft buf))
-	  (setq draft buf)))
-    draft))
+  (let* ((draft-dir (file-name-as-directory mew-draft-folder))
+	 (regex (mew-folder-regex draft-dir))
+         (bufs (mew-buffer-list regex))
+	 (msgregex (format "\\([0-9]+\\)\\(%s\\)?$" mew-suffix))
+         nums n)
+    (setq nums (mapcar (lambda (x) (string-match msgregex x) (string-to-number (mew-match-string 1 x))) bufs))
+    (dolist (num nums)
+      (if (or (null n) (< n num))
+	  (setq n num)))
+    (and n (format "%s%d%s" draft-dir n (if mew-use-suffix mew-suffix "")))))
 
 (defun mew-summary-jump-to-draft-buffer ()
   "Jump to one of drafts if exists."
@@ -202,24 +218,27 @@ the message is then displayed."
 
 (defun mew-summary-jump-top ()
   "Go to the beginning of this Summary mode.
-If 'mew-summary-jump-top-then-display' is non-nil, 
+If 'mew-summary-jump-top-then-display' is non-nil,
 the top message is then displayed."
   (interactive)
   (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-min))
   (if mew-summary-jump-top-then-display
-      (mew-summary-display nil)))
+      (mew-summary-display)))
 
 (defun mew-summary-jump-bottom ()
   "Go to the end of this Summary mode.
-If 'mew-summary-jump-bottom-then-display' is non-nil, 
+If 'mew-summary-jump-bottom-then-display' is non-nil,
 the top message is then displayed."
   (interactive)
   (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-max))
-  (unless (bobp) (forward-line -1))
+  (if (and (mew-summary-exclusive-p 'no-msg)
+	   (not (bobp))
+	   (mew-sinfo-get-disp-msg))
+      (forward-line -1))
   (if mew-summary-jump-bottom-then-display
-      (mew-summary-display nil)))
+      (mew-summary-display)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -227,35 +246,11 @@ the top message is then displayed."
 ;;;
 
 (defun mew-summary-exchange-point ()
-  "Get back to the position before typing '\\<mew-summary-mode-map>\\[mew-summary-retrieve]'."
+  "\\<mew-summary-mode-map>Get back to the position before typing '\\[mew-summary-retrieve]' or '\\[mew-summary-ls]'."
   (interactive)
   (mew-summary-only
-   (cond
-    ((not (mew-folder-inboxp (mew-summary-folder-name 'ext)))
-     (message "Cannot be used in this folder"))
-    ((not (marker-position mew-summary-inbox-position))
-     (message "No previous folder"))
-    (t
-     (let* ((buf (marker-buffer mew-summary-inbox-position))
-	    (pos (marker-position mew-summary-inbox-position))
-	    (folder (mew-summary-folder-name 'ext))
-	    getback)
-       (cond
-	((equal (current-buffer) buf)
-	 (goto-char pos))
-	((not (get-buffer buf))
-	 (message "Previous folder not exist"))
-	((mew-folder-virtualp folder)
-	 (if (mew-virtual-thread-p folder)
-	     (if (mew-thread-cache-valid-p folder)
-		 (setq getback t)
-	       (message "%s is old" folder))
-	   (setq getback t)))
-	(t
-	 (setq getback t)))
-       (when getback
-	 (switch-to-buffer buf)
-	 (goto-char pos)))))))
+   (let ((pos (mew-sinfo-get-ret-pos)))
+     (if pos (goto-char pos)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -302,7 +297,7 @@ the top message is then displayed."
 If called with '\\[universal-argument]', the body of the message
 \(excluding its header) is sent."
   (interactive
-   (list current-prefix-arg 
+   (list current-prefix-arg
 	 (read-string "Shell command on message: " mew-last-shell-command)))
   (mew-summary-display 'redisplay)
   (when (or (not mew-ask-pipe)
@@ -328,17 +323,17 @@ If called with '\\[universal-argument]', the body of the message
 If called with '\\[universal-argument]', you can specify a printer name."
   (interactive "P")
   (mew-summary-display 'redisplay)
-  (when (y-or-n-p "Print this message? ")
-    (let ((printer-name (if arg
-			    (read-string "Printer name: ")
-			  printer-name))
-	  have-header str)
-      (save-excursion
-	(set-buffer (mew-buffer-message))
-	(save-restriction
-	  (widen)
-	  (setq have-header (mew-msghder-p))
-	  (setq str (mew-buffer-substring (point-min) (point-max)))))
+  (let ((printer-name (if arg
+			  (read-string "Printer name: ")
+			printer-name))
+	have-header str)
+    (save-excursion
+      (set-buffer (mew-buffer-message))
+      (save-restriction
+	(widen)
+	(setq have-header (mew-msghdr-p))
+	(setq str (buffer-substring (point-min) (point-max)))))
+    (when (y-or-n-p "Print this message? ")
       (with-temp-buffer
 	(insert str)
 	(if have-header
@@ -362,11 +357,9 @@ If called with '\\[universal-argument]', you can specify a printer name."
        (with-temp-buffer
 	 (insert xface)
 	 (mew-x-face-compface-to-xbm)
-	 (mew-frwlet
-	  mew-cs-dummy mew-cs-text-for-write
-	  (write-region (point-min) (point-max) file nil 'no-msg)))
-       (mew-mime-start-process
-	mew-prog-image/*-ext mew-prog-image/*-ext-arg file)))))
+	 (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	   (write-region (point-min) (point-max) file nil 'no-msg)))
+       (mew-mime-image-ext file)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -380,25 +373,25 @@ argument."
   (mew-summary-msg-or-part
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number))
-	  (file (mew-expand-folder fld msg))
-	  (command (read-string "Command: ")))
-     (while (not (mew-which-exec command))
-       (setq command (read-string "Command: ")))
-     (message "Executing %s for %s..." command msg)
-     (call-process command nil nil nil file)
-     (message "Executing %s for %s...done" command msg))))
+	  (file (mew-expand-msg fld msg))
+	  (cmd-args (mew-input-command))
+	  (cmd (car cmd-args))
+	  (args (nconc (cdr cmd-args) (list file))))
+     (message "Executing %s for %s..." cmd msg)
+     (apply 'call-process cmd nil nil nil args)
+     (message "Executing %s for %s...done" cmd msg))))
 
 (defun mew-summary-cmd-msgs ()
   "Executing an external command specifying messages
 marked with '*' as arguments."
   (interactive)
   (mew-summary-multi-msgs
-   (let ((command (read-string "Command: ")))
-     (while (not (mew-which-exec command))
-       (setq command (read-string "Command: ")))
-     (message "Executing %s ..." command)
-     (apply 'call-process command nil nil nil FILES)
-     (message "Executing %s ...done" command))))
+   (let* ((cmd-args (mew-input-command))
+	  (cmd (car cmd-args))
+	  (args (nconc (cdr cmd-args) FILES)))
+     (message "Executing %s ..." cmd)
+     (apply 'call-process cmd nil nil nil args)
+     (message "Executing %s ...done" cmd))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -418,194 +411,6 @@ marked with '*' as arguments."
 	    (mew-draft-cite arg))
 	(message "No such draft buffer!")))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Executing internal commands
-;;;
-
-(defun mew-summary-ask-ct (ct fname)
-  (let (pt fl)
-    (cond
-     (fname
-      (setq ct (or (mew-ctdb-ct (mew-ctdb-by-file fname))
-		   (mew-content-type (mew-sinfo-get-case))))
-      (setq pt "Type for %s (%s): ")
-      (setq fl fname))
-     (t
-      (setq pt "Type %s(%s): ")
-      (setq fl "")))
-    (mew-input-type pt fl ct mew-mime-content-type-list)))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Executing internal commands
-;;;
-
-(defun mew-summary-execute-internal ()
-  "Visualize MIME parts in Message mode.
-This is useful, for example, an image file attached but
-whose Content-Type: is Application/Octet-Stream. With this
-command, you can specify its Content-Type: correctly and
-visualize it in Message mode."
-  (interactive)
-  (let* ((fld (mew-summary-folder-name))
-	 (msg (mew-summary-message-number2))
-	 (nums (mew-syntax-nums))
-	 (cache (mew-cache-hit fld msg 'must-hit))
-	 (syntax (mew-cache-decode-syntax cache))
-	 (stx (mew-syntax-get-entry syntax nums))
-	 (ctl (mew-syntax-get-ct stx))
-	 (ct (mew-syntax-get-value ctl 'cap))
-	 (mew-inherit-ct ct)
-	 (cte (mew-syntax-get-cte stx))
-	 (begin (mew-syntax-get-begin stx))
-	 (end (mew-syntax-get-end stx))
-	 (params (mew-syntax-get-params ctl))
-	 (cdpl (mew-syntax-get-cdp stx))
-	 (fname (mew-syntax-get-filename cdpl ctl))
-	 (win (selected-window))
-	 program)
-    (setq ct (mew-summary-ask-ct ct fname))
-    (setq program (mew-ctdb-prog (mew-ctdb-by-ct ct)))
-    (mew-summary-toggle-disp-msg 'on)
-    (mew-window-configure 'message)
-    ;; message buffer
-    (unwind-protect
-	(mew-elet
-	  (mew-erase-buffer)
-	  (cond
-	   ((and (listp program) (nth 0 program)
-		 (symbolp (nth 0 program)) (fboundp (nth 0 program)))
-	    (if (or (mew-ct-imagep ct) (mew-ct-modelp ct))
-		(funcall (nth 0 program) cache begin end params fname ct cte)
-	      (funcall (nth 0 program) cache begin end params)))
-	   ;; This is not possible since the part may be analyzed as
-	   ;; a single part, not a message.
-	   ;; ((and (symbolp program) (eq program 'mew-mime-message/rfc822))
-	   ;; (funcall program cache stx))
-	   (t
-	    (message "Not supported yet")))
-	  (mew-summary-display-postscript 'no-hook))
-      (select-window win))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Executing external commands
-;;;
-
-(defun mew-summary-execute-external (&optional arg)
-  "Execute an external command for the current part according to Content-Type:. 
-If a body is not displayed as a part, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again-alternative]' to display the body as a part. 
-Then choose the part and execute this command.
-If called with '\\[universal-argument]', Content-Type: is asked."
-  (interactive "P")
-  (mew-summary-msg-or-part
-   (mew-summary-execute-base arg nil nil)))
-
-(defun mew-summary-execute-command (&optional arg)
-  "Execute an inputted command for the current part.
-If a body is not displayed as a part, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again-alternative]' to display the body as a part. 
-Then choose the part and execute this command.
-If called with '\\[universal-argument]', options are asked."
-  (interactive "P")
-  (mew-summary-msg-or-part
-   (mew-summary-execute-base
-    nil
-    (read-string "Command: ")
-    (if arg (delete "" (mew-split-quoted
-			(read-string "Options: ") mew-sp ?\" ?\"))))))
-
-(defun mew-summary-execute-base (ask-type com args)
-  (let* ((fld (mew-summary-folder-name))
-	 (msg (mew-summary-message-number2))
-	 (nums (mew-syntax-nums))
-	 (cache (mew-cache-hit fld msg 'must-hit))
-	 (syntax (mew-cache-decode-syntax cache))
-	 (stx (mew-syntax-get-entry syntax nums))
-	 (ctl (mew-syntax-get-ct stx))
-	 (ct (mew-syntax-get-value ctl 'cap))
-	 (mew-inherit-ct ct)
-	 (begin (mew-syntax-get-begin stx))
-	 (end (mew-syntax-get-end stx))
-	 (params (mew-syntax-get-params ctl))
-	 (cdpl (mew-syntax-get-cdp stx))
-	 (fname (mew-syntax-get-filename cdpl ctl))
-	 (win (selected-window))
-	 program)
-    (when ask-type
-      (setq ct (mew-summary-ask-ct ct fname)))
-    (setq program (or com (mew-ctdb-prog (mew-ctdb-by-ct ct))))
-    (mew-summary-toggle-disp-msg 'on)
-    (mew-window-configure 'message)
-    ;; message buffer
-    (unwind-protect
-	(mew-elet
-	  (mew-summary-execute-base1 program ct ctl cache begin end params fname args)
-	  (mew-summary-display-postscript 'no-hook))
-      (select-window win))))
-
-(defun mew-summary-execute-base1 (program ct ctl cache begin end params fname args)
-  (let (options async)
-    (if (and (listp program) (nth 1 program) (symbolp (nth 1 program)))
-	(cond
-	 ((not (fboundp (nth 1 program)))
-	  (mew-message-for-summary "Use '\\[mew-summary-execute-command]' or \\[universal-argument] \\[mew-summary-execute-external]'"))
-	 ((or (mew-ct-imagep ct) (mew-ct-modelp ct))
-	  (funcall (nth 1 program) cache begin end params fname))
-	 (t
-	  (funcall (nth 1 program) cache begin end params)))
-      (if (listp program)
-	  (mew-set '(program options async) program)
-	(setq options args)
-	(setq async t))
-      (cond
-       ((symbolp program)
-	(message "Function %s is not executable here" program))
-       ((not (mew-which-exec program))
-	(message "Program %s is not found" program))
-       (t
-	(mew-summary-execute-base2 program ct ctl cache begin end params fname options async))))))
-
-(defun mew-summary-execute-base2 (program ct ctl cache begin end params fname options async)
-  (let ((file (mew-make-temp-name fname))
-	wcs)
-    (save-excursion
-      (set-buffer cache)
-      ;; NEVER use call-process-region for privacy reasons
-      (cond
-       ((not (mew-ct-linebasep ct))
-	(setq wcs mew-cs-binary))
-       ((not (mew-ct-textp ct))
-	(setq wcs mew-cs-text-for-write))
-       (t
-	(cond
-	 ((string= mew-ct-htm ct)
-	  (setq wcs (mew-text/html-detect-cs begin end ctl))
-	  (unless (mew-coding-system-p wcs)
-	    (setq wcs (mew-charset-to-cs
-		       (mew-syntax-get-param ctl "charset")))))
-	 ((string= mew-ct-xml ct)
-	  (setq wcs (mew-text/xml-detect-cs begin end ctl))
-	  (unless (mew-coding-system-p wcs)
-	    (setq wcs (mew-charset-to-cs
-		       (mew-syntax-get-param ctl "charset")))))
-	 
-	 (t
-	  (setq wcs (mew-charset-to-cs
-		     (mew-syntax-get-param ctl "charset")))))
-	(unless (mew-coding-system-p wcs)
-	  (setq wcs (if mew-decode-broken
-			(mew-charset-to-cs
-			 (mew-charset-guess-region
-			  begin end))
-		      mew-cs-text-for-write)))))
-      (mew-frwlet
-       mew-cs-dummy wcs
-       (write-region begin end file nil 'no-msg))
-      (if async
-	  (mew-mime-start-process program options file)
-	(mew-mime-call-process program options file)))))
-  
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Flushing the queue
@@ -626,8 +431,8 @@ If executed with '\\[universal-argument]', you can set the sending case."
 	   (proto (mew-proto-to-flush fld))
 	   qfld case)
       (if (or arg mew-ask-flush-case)
-	  (setq case (mew-input-case mew-case-output "Queue"))
-	(setq case mew-case-output))
+	  (setq case (mew-input-case mew-case "Queue"))
+	(setq case mew-case))
       (if (or (mew-folder-queuep fld) (mew-folder-postqp fld))
 	  (progn
 	    (setq qfld fld)
@@ -650,7 +455,7 @@ If executed with '\\[universal-argument]', you can set the sending case."
 (defun mew-summary-kill-subprocess (&optional kill-ssx)
   "\\<mew-summary-mode-map>
 Kill a process in Summary mode.
-Sometime a process accidentally remains in Summary mode. 
+Sometime a process accidentally remains in Summary mode.
 In this situation, you cannot execute '\\[mew-summary-retrieve]', '\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.
 Use this command to solve this problem.
 
@@ -663,13 +468,16 @@ any, are killed."
     (if kill-ssx
 	(progn
 	  (setq pros (process-list))
-	  (while pros
-	    (if (string-match "mew" (process-name (car pros)))
-		(delete-process (car pros)))
-	    (setq pros (cdr pros)))
+	  (dolist (pro pros)
+	    (if (string-match "mew" (process-name pro))
+		(delete-process pro)))
 	  (if disp-msg (message "All SSH/SSL/TLS processes were killed")))
       (cond
        ((not (processp process))
+	;; If you type "x" over SSH and type "C-g" to quit,
+	;; refile information remains invisible.
+	;; This command turns it visible.
+	(mew-summary-visible-buffer (current-buffer))
 	(if disp-msg (message "No process to kill. This buffer is unlocked anyway")))
        ((eq process t)
 	;; Emacs is locking
@@ -718,19 +526,17 @@ any, are killed."
 	   (case-fold-search nil)
 	   (files FILES) start)
        (with-temp-buffer
-	 (mew-frwlet
-	  mew-cs-text-for-read mew-cs-text-for-write
-	  (while files
-	    (mew-erase-buffer)
-	    (mew-insert-file-contents (car files))
-	    (goto-char (point-min))
-	    (if (re-search-forward mew-eoh nil t)
-		(forward-line))
-	    (setq start (point))
-	    (goto-char (point-max))
-	    (unless (bolp) (insert "\n"))
-	    (write-region start (point-max) tmp 'append 'no-msg)
-	    (setq files (cdr files))))
+	 (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	   (dolist (file files)
+	     (mew-erase-buffer)
+	     (mew-insert-file-contents file)
+	     (goto-char (point-min))
+	     (if (re-search-forward mew-eoh nil t)
+		 (forward-line))
+	     (setq start (point))
+	     (goto-char (point-max))
+	     (unless (bolp) (insert "\n"))
+	     (write-region start (point-max) tmp 'append 'no-msg)))
 	 (mew-erase-buffer)
 	 (cd dir)
 	 ;; mew-folder-local is dummy to let mewencode use
@@ -777,22 +583,21 @@ message."
 	   (folder (mew-input-burst-folder))
 	   (tfile (mew-make-temp-name))
 	   (targets FLD-MSG-LIST)
-	   fld-msg num ct med dbl vec len idx ttl TTL fid FID i ent
+	   num ct med dbl vec len idx ttl TTL fid FID i
 	   beg regex)
        (message "Joining...")
        (with-temp-buffer
 	 (mew-set-buffer-multibyte t)
-	 (insert "CD: " cfld "\n")
-	 (while targets
-	   (setq fld-msg (car targets))
-	   (setq targets (cdr targets))
-	   (insert (cdr fld-msg) "\n"))
-	 (mew-frwlet
-	  mew-cs-text-for-read mew-cs-text-for-write
-	  (write-region (point-min) (point-max) tfile nil 'no-msg))
+	 (insert "CD: " (mew-expand-folder2 cfld) "\n")
+	 (let ((default-directory (mew-expand-folder cfld)))
+	   (dolist (fld-msg targets)
+	     (insert (mew-msg-get-filename (cdr fld-msg)) "\n")))
+	 (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	   (write-region (point-min) (point-max) tfile nil 'no-msg))
 	 (mew-erase-buffer)
 	 (call-process mew-prog-mewl nil t nil
 		       "-b" mew-mail-path "-l" "0"
+		       "-x" mew-suffix
 		       "-i" tfile "-d" "content-type")
 	 (mew-delete-file tfile)
 	 (goto-char (point-min))
@@ -808,9 +613,7 @@ message."
 	     (setq dbl (cons (cons num ct) dbl))))
 	 (setq len (length dbl))
 	 (setq vec (make-vector len nil))
-	 (while dbl
-	   (setq ent (car dbl))
-	   (setq dbl (cdr dbl))
+	 (dolist (ent dbl)
 	   (setq num (car ent))
 	   (setq ttl (mew-syntax-get-param ent "total"))
 	   (setq fid (mew-syntax-get-param ent "id"))
@@ -824,65 +627,66 @@ message."
 		   (error "fragment id mismatch"))
 	     (setq FID fid))
 	   (unless idx (error "no number"))
-	   (setq idx (1- (string-to-int idx)))
+	   (setq idx (1- (string-to-number idx)))
 	   (if (and (>= idx 0) (< idx len))
 	       (aset vec idx num)
 	     (error "invalid number")))
 	 (unless TTL (error "no total"))
-	 (setq i 0)
-	 (while (< i len)
-	   (unless (stringp (aref vec i)) (error "Not enough fragments"))
-	   (setq i (1+ i)))
+	 (dotimes (< i len)
+	   (unless (stringp (aref vec i)) (error "Not enough fragments")))
 	 ;; now reassemble
 	 (mew-erase-buffer)
-	 (mew-frwlet
-	  mew-cs-text-for-read mew-cs-text-for-write
-	  ;; the first fragment
-	  (goto-char (point-max))
-	  (save-restriction
-	    (narrow-to-region (point) (point))
-	    (mew-insert-file-contents (mew-expand-folder cfld (aref vec 0)))
-	    ;; Removing unnecessary fields from the encapsulating
-	    ;; (outer) header.
-	    (goto-char (point-min))
-	    (mew-header-delete-lines mew-field-delete-for-joining)
-	    ;; Concatenating the two headers.
-	    (goto-char (point-min))
-	    (re-search-forward mew-eoh nil t)
-	    (setq beg (point))
-	    (while (looking-at mew-eoh) (forward-line))
-	    (delete-region beg (point))
-	    ;; Removing unnecessary fields from the encapsulated
-	    ;; (inner) header.
-	    (setq beg (point))
-	    (when (re-search-forward mew-eoh nil t)
-	      (setq regex (mew-make-field-regex mew-field-delete-for-joining))
-	      (save-restriction
-		(narrow-to-region beg (point))
-		(goto-char (point-min))
-		(let ((case-fold-search t))
-		  (while (not (eobp))
-		    (if (looking-at regex)
-			(setq beg nil) ;; logic is reversed
-		      (setq beg (point)))
-		    (forward-line)
-		    (mew-header-goto-next)
-		    (if beg (delete-region beg (point))))))))
-	  ;; the second and subsequent fragments.
-	  (setq i 1)
-	  (while (< i len)
-	    (goto-char (point-max))
-	    (save-restriction
-	      (narrow-to-region (point) (point))
-	      (mew-insert-file-contents (mew-expand-folder cfld (aref vec i)))
-	      (goto-char (point-min))
-	      (re-search-forward mew-eoh nil t)
-	      (forward-line)
-	      (delete-region (point-min) (point)))
-	    (setq i (1+ i)))
-	  (write-region (point-min) (point-max)
-			(mew-folder-new-message folder)
-			nil 'no-msg)))
+	 (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	   ;; the first fragment
+	   (goto-char (point-max))
+	   (save-restriction
+	     (narrow-to-region (point) (point))
+	     (mew-insert-file-contents (mew-expand-msg cfld (aref vec 0)))
+	     ;; Removing unnecessary fields from the encapsulating
+	     ;; (outer) header.
+	     (goto-char (point-min))
+	     (mew-header-delete-lines mew-field-delete-for-joining)
+	     ;; Concatenating the two headers.
+	     (goto-char (point-min))
+	     (re-search-forward mew-eoh nil t)
+	     (setq beg (point))
+	     (while (looking-at mew-eoh)
+	       (forward-line)
+	       (if (eobp) (error "invalid message (number 1)")))
+	     (delete-region beg (point))
+	     ;; Removing unnecessary fields from the encapsulated
+	     ;; (inner) header.
+	     (setq beg (point))
+	     (when (re-search-forward mew-eoh nil t)
+	       (setq regex (mew-make-field-regex mew-field-delete-for-joining))
+	       (save-restriction
+		 (narrow-to-region beg (point))
+		 (goto-char (point-min))
+		 (let ((case-fold-search t))
+		   (while (not (eobp))
+		     (if (looking-at regex)
+			 (setq beg nil) ;; logic is reversed
+		       (setq beg (point)))
+		     (forward-line)
+		     (mew-header-goto-next)
+		     (if beg (delete-region beg (point))))))))
+	   ;; the second and subsequent fragments.
+	   (setq i 1)
+	   (while (< i len)
+	     (goto-char (point-max))
+	     (save-restriction
+	       (narrow-to-region (point) (point))
+	       (mew-insert-file-contents (mew-expand-msg cfld (aref vec i)))
+	       (goto-char (point-min))
+	       (re-search-forward mew-eoh nil t)
+	       (forward-line)
+	       (if (eobp) (error "invalid message (number %d)" (1+ i)))
+	       (delete-region (point-min) (point)))
+	     (setq i (1+ i)))
+	   (mew-header-delete-lines `(,mew-x-mew-uidl:))
+	   (write-region (point-min) (point-max)
+			 (mew-folder-new-message folder)
+			 nil 'no-msg)))
        (mew-touch-folder folder)
        (message "Joining...done")
        (when mew-ask-folder-after-join
@@ -912,21 +716,29 @@ message."
 	(message "Nothing was processed")
       (setq case (mew-case:folder-case case:trash))
       (setq trash (mew-case:folder-folder case:trash))
-      (setq trashdir (mew-expand-folder trash))
+      (setq trashdir (mew-expand-folder case:trash))
       (setq msgs (mew-dir-messages trashdir))
       (if (and (mew-folder-localp trash) (null msgs))
 	  (message "No messages to be removed in %s" trash)
 	(if (not (y-or-n-p (format "Remove all messages in %s? " case:trash)))
 	    (message "No messages removed in %s" trash)
+	  (if (string= folder trash) (mew-summary-reset))
 	  (message "Removing all messages in %s..." case:trash)
+	  (when (get-buffer case:trash)
+	    (save-excursion
+	      (set-buffer case:trash)
+	      (mew-summary-kill-subprocess)))
 	  (mew-summary-unlink-msgs case:trash msgs)
 	  (mew-summary-folder-cache-clean case:trash)
 	  (message "Removing all messages in %s...done" case:trash)
 	  (when (mew-folder-imapp trash)
 	    (if (get-buffer case:trash)
 		(set-buffer case:trash)
-	      (mew-summary-switch-to-folder case:trash nil 'set-buffer))
-	    (mew-imap-retrieve case 'exec case:trash '(("1:*") nil nil nil 0 -1))))))))
+	      (mew-summary-switch-to-folder case:trash 'set-buffer))
+	    (mew-imap-retrieve case 'exec case:trash
+			       (mew-make-execinfo :remove '("1:*")
+						  :rttl   0
+						  :dttl   -1))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -934,7 +746,7 @@ message."
 ;;;
 
 (defun mew-summary-local-copy ()
-  "Copy a cached message in a remote folder to a local folder."
+  "Copy a message to a local folder."
   (interactive)
   (mew-summary-msg-or-part
    (let ((fld (mew-summary-folder-name))
@@ -943,11 +755,11 @@ message."
      (save-excursion
        (mew-summary-goto-message)
        (setq msg (mew-summary-message-number)))
-     (setq file (mew-expand-folder fld msg))
+     (setq file (mew-expand-msg fld msg))
      (setq dstfld (mew-input-local-folder mew-inbox-folder))
      (when (and dstfld (mew-local-folder-check dstfld 'ask))
        (setq dstmsg (mew-folder-new-message dstfld 'numonly))
-       (setq dstfile (mew-expand-folder dstfld dstmsg))
+       (setq dstfile (mew-expand-msg dstfld dstmsg))
        (when (get-buffer dstfld)
 	 (set-buffer dstfld)
 	 (setq exclusivep (mew-summary-exclusive-p)))
@@ -962,9 +774,10 @@ message."
 	 (message "Copied to %s/%s" dstfld dstmsg)))))))
 
 (defun mew-summary-mark-local-copy (&optional arg)
-  "Copy cached message marked with '*' in a remote folder to a local
-folder."
-  (interactive "P")
+  "Copy messages marked with '*' to a local folder.  If called
+with '\\[universal-argument]', only messages marked with '*' in
+the region are handled."
+ (interactive "P")
   (mew-summary-not-in-draft
    (let ((mew-use-highlight-x-face nil)
 	 (fld (mew-summary-folder-name))
@@ -982,7 +795,8 @@ folder."
        (setq end (point-max))))
      (setq dstfld (mew-input-local-folder mew-inbox-folder))
      (when (and dstfld (mew-local-folder-check dstfld 'ask))
-       (setq dstmsg (mew-folder-new-message dstfld 'numonly))
+       (let ((mew-use-suffix nil))
+	 (setq dstmsg (mew-folder-new-message dstfld 'numonly)))
        (setq msgs (mew-summary-mark-collect mew-mark-review beg end))
        (when (get-buffer dstfld)
 	 (save-excursion
@@ -997,26 +811,24 @@ folder."
 	 (message "Copying...")
 	 (setq total (length msgs))
 	 (setq i 1)
-	 (while msgs
-	   (setq file (mew-expand-folder fld (car msgs)))
-	   (setq dstfile (mew-expand-folder dstfld dstmsg))
+	 (dolist (msg msgs)
+	   (setq file (mew-expand-msg fld msg))
+	   (setq dstfile (mew-expand-new-msg dstfld dstmsg))
 	   (message "Copying (%d/%d)..." i total)
 	   (mew-summary-local-copy-one file dstfile)
 	   (setq dstmsg (number-to-string (1+ (string-to-number dstmsg))))
-	   (setq msgs (cdr msgs))
 	   (setq i (1+ i)))
 	 (mew-touch-folder dstfld)
 	 (message "Copying...done")))))))
 
 (defun mew-summary-local-copy-one (srcfile dstfile)
   (with-temp-buffer
-    (mew-frwlet
-     mew-cs-text-for-read mew-cs-text-for-write
-     (mew-insert-file-contents srcfile)
-     (goto-char (point-min))
-     (mew-header-delete-lines (list mew-x-mew-uidl:))
-     (write-region (point-min) (point-max) dstfile nil 'no-msg)
-     (mew-set-file-modes dstfile))))
+    (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+      (mew-insert-file-contents srcfile)
+      (goto-char (point-min))
+      (mew-header-delete-lines (list mew-x-mew-uidl:))
+      (write-region (point-min) (point-max) dstfile nil 'no-msg)
+      (mew-set-file-modes dstfile))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1040,7 +852,7 @@ folder."
 	 (when (y-or-n-p "Delete this folder? ")
 	   (mew-delete-directory-recursively dir)
 	   (mew-local-folder-delete folder)
-	   (mew-nmz-folder-index-delete folder)
+	   (run-hook-with-args 'mew-summary-delete-folder-hook folder)
 	   (mew-summary-visit-folder mew-inbox-folder)
 	   (mew-kill-buffer buf))))
       ((mew-folder-imapp folder)
@@ -1057,7 +869,7 @@ folder."
 		 (setq mew-summary-buffer-process-error nil))
 	     (mew-imap-folder-delete case folder)
 	     (mew-delete-directory-recursively dir) ;; cache
-	     (mew-nmz-folder-index-delete case:folder)
+	     (run-hook-with-args 'mew-summary-delete-folder-hook case:folder)
 	     (mew-summary-visit-folder mew-imap-inbox-folder)
 	     (mew-kill-buffer buf)))))))))
 
@@ -1080,9 +892,9 @@ folder."
 	((string= folder new-folder)
 	 (message "%s was not moved" folder))
 	((file-exists-p new-dir)
-	 (message "%s was not moved so %s is existed" folder new-folder))
+	 (message "%s was not moved since %s exists" folder new-folder))
 	(t
-	 (rename-file dir new-dir)
+	 (mew-rename-directory dir new-dir)
 	 (mew-summary-switch-to-folder new-folder)
 	 (mew-kill-buffer buf)
 	 (if (mew-folder-node-p folder)
@@ -1090,7 +902,7 @@ folder."
 	     (mew-local-update 'force)
 	   (mew-local-folder-delete folder)
 	   (mew-local-folder-insert new-folder))
-	 (mew-nmz-folder-index-rename folder new-folder))))
+	 (run-hook-with-args 'mew-summary-rename-folder-hook folder new-folder))))
       ((mew-folder-imapp folder)
        (setq new-folder (car (mew-input-refile-folders (list folder) t case mew-folder-imap)))
        (setq case:new-folder (mew-case-folder case new-folder))
@@ -1099,7 +911,7 @@ folder."
 	((string= folder new-folder)
 	 (message "%s was not moved" folder))
 	((file-exists-p new-dir)
-	 (message "%s was not moved so %s is existed" folder new-folder))
+	 (message "%s was not moved since %s exists" folder new-folder))
 	(t
 	 (mew-imap-retrieve case 'rename case:folder case:new-folder)
 	 (mew-timing)
@@ -1112,12 +924,13 @@ folder."
 	   (mew-summary-switch-to-folder case:new-folder)
 	   (mew-kill-buffer buf)
 	   (if (mew-folder-node-p folder case)
-	       (if (y-or-n-p "Collect a folder list now? ")
+	       (if (y-or-n-p "Collect folders now? ")
 		   (mew-imap-update case)
-		 (mew-message-for-summary "Type '3\\[mew-status-update]' to collect a folder list"))
+		 (mew-message-for-summary "Type '\\[universal-argument]\\[mew-status-update]' to collect folders"))
 	     (mew-imap-folder-delete case folder)
 	     (mew-imap-folder-insert case new-folder))
-	   (mew-nmz-folder-index-rename case:folder case:new-folder)))))))))
+	   (run-hook-with-args 'mew-summary-rename-folder-hook
+			       case:folder case:new-folder)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1130,11 +943,11 @@ folder."
 ;;;
 
 (defun mew-summary-toggle-disp-msg (&optional arg)
-  "Toggle 'Summary mode only' and 'Summary & Message mode'. If 
+  "Toggle 'Summary mode only' and 'Summary & Message mode'. If
 you choose 'Summary mode only', you can quickly put the delete
 	marks since the next message is not displayed."
   (interactive)
-  (cond 
+  (cond
    ((eq arg 'on)
     (mew-sinfo-set-disp-msg t))
    ((eq arg 'off)
@@ -1245,7 +1058,7 @@ See also 'mew-spam-prog' and 'mew-spam-prog-args."
   (mew-summary-msg
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number))
-	  (file (mew-expand-folder fld msg))
+	  (file (mew-expand-msg fld msg))
 	  (case (mew-sinfo-get-case))
 	  (prog (mew-spam-prog case))
 	  (args (mew-spam-prog-args case))
@@ -1269,7 +1082,7 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
   (mew-summary-msg
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number))
-	  (file (mew-expand-folder fld msg))
+	  (file (mew-expand-msg fld msg))
 	  (case (mew-sinfo-get-case))
 	  (prog (mew-ham-prog case))
 	  (args (mew-ham-prog-args case))
@@ -1292,23 +1105,72 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
 ;;;
 
 (defun mew-summary-toggle-invisible ()
+  "Toggle the value of search-invisible.
+Invisible information in Summary mode cannot be searched by default.
+Executing this command enables searching such information."
   (interactive)
   (if (eq search-invisible t)
       (setq search-invisible 'open)
     (setq search-invisible t))
   (message "'search-invisible' is set to '%s'" search-invisible))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Lines
+;;;
+
+(defun mew-summary-line ()
+  "Toggle normal lines, wrapped lines and long lines."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-line)
+	(select-window cwin)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Horizontal Scrolling
+;;;
+
+(defun mew-summary-scroll-left ()
+  "Scroll the Message window to the right."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-scroll-left)
+	(select-window cwin)))))
+
+(defun mew-summary-scroll-right ()
+  "Scroll the Message window to the left."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-scroll-right)
+	(select-window cwin)))))
+
 (provide 'mew-summary4)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1317,7 +1179,7 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
